
(define Pi (* 4 (atan 1.0)))
(define I 0+1i)

(define-macro (seq i b n expr)
  `(map (lambda (,i) ,expr) (.. ,b ,n)))

(define-macro (sum i b n expr)
  `(apply + (seq ,i ,b ,n ,expr)))

(define (DFT v)
  (with n (length v)
    (seq i 0 n
         (sum k 0 n
              (* (list-ref v k) (exp (/ (* 2 Pi I i k) n)))))))

(define (g x a s)
  (exp (- (/ (* Pi s s x x) (* a a)))))

(define (G x a s)
  (sum j -10 10
       (g (+ x j) a s)))

(define (f x u a)
  (with s (length u)
    (sum j 0 s
         (* (list-ref u j)
            (G (- x (/ j s)) a s)))))

(define-macro ($points u w dx)
  `($for (_k_ (.. 0 (length ,u)))
     ($point (+ ,dx (* (/ _k_ (length ,u)) ,w))
             (real-part (list-ref ,u _k_)))))

(define-macro ($axes x1 x2 y1 y2 dx)
  `($graphical
     ($line ($point (+ ,dx ,x1) 0) ($point (+ ,dx ,x2) 0))
     ($line ($point ,dx ,y1) ($point ,dx ,y2))))

(define-macro ($show-fft u w pad)
  `($let* ((u ,u)
           (u^ (map real-part (DFT u)))
           (w ,w)
           (pad ,pad)
           (y1 (apply min (append u u^)))
           (y2 (apply max (append u u^)))
           (dx1 (- (- (- w) 1.0) pad))
           (dx2 2.0))
     ($graphical
      ($axes (- -0.5 pad) (+ w pad 0.5)
             (- y1 0.5) (+ y2 0.5) dx1)
      ($axes -0.5 (+ w 0.5) (- y1 0.5) (+ y2 0.5) dx2)
      ($pen-color "dark grey"
        ($line ($point (+ w dx1) (- y1 0.5))
               ($point (+ w dx1) (+ y2 0.5))))
      ($pen-color "dark grey"
        ($line ($point (+ w dx2) (- y1 0.5))
               ($point (+ w dx2) (+ y2 0.5))))
      ($pen-color "red"
        ($points u w dx1)
        ($points u^ w dx2)))))

(define-macro ($show-fft* u w pad)
  `($let* ((u ,u)
           (u^ (map real-part (DFT u)))
           (w ,w)
           (pad ,pad)
           (y1 (apply min (append u u^)))
           (y2 (apply max (append u u^)))
           (dx1 (- (- (- w) 1.0) pad))
           (dx2 2.0))
     ($graphical
      ($axes (- -0.5 pad) (+ w pad 0.5)
             (- y1 0.5) (+ y2 0.5) dx1)
      ($pen-color "dark grey"
        ($line ($point (+ w dx1) (- y1 0.5))
               ($point (+ w dx1) (+ y2 0.5))))
      ($pen-color "red"
        ($points u w dx1)))))

(define-macro ($smoothed u a w pad dx)
  `($graph2d (+ (- ,pad) ,dx) (+ ,w ,pad ,dx) 200
     (lambda (x) (f (/ (- x ,dx) ,w) ,u ,a))))

(define-macro ($show-smoothed u a t w pad)
  `($let* ((u ,u)
           (a ,a)
           (t ,t)
           (U (seq i 0 t (f (/ i t) u a)))
           (U~ (map real-part (DFT U)))
           (U^ (map (lambda (y) (/ (* y (length u)) t)) U~))
           (w ,w)
           (pad ,pad)
           (y1 (apply min (append U U^)))
           (y2 (apply max (append U U^)))
           (dx1 (- (- (- w) 1.0) pad))
           (dx2 2.0))
     ($graphical
      ($axes (- -0.5 pad) (+ w pad 0.5)
             (- y1 0.5) (+ y2 0.5) dx1)
      ($axes -0.5 (+ w 0.5)
             (- y1 0.5) (+ y2 0.5) dx2)
      ($pen-color "dark grey"
        ($line ($point (+ w dx1) (- y1 0.5))
               ($point (+ w dx1) (+ y2 0.5))))
      ($pen-color "dark grey"
        ($line ($point (+ w dx2) (- y1 0.5))
               ($point (+ w dx2) (+ y2 0.5))))
      ($pen-color "magenta"
        ($smoothed u a w pad dx1))
      ($pen-color "red"
        ($points U w dx1)
        ($points U^ w dx2)))))

(define-macro ($show-smoothed* u a t w pad)
  `($let* ((u ,u)
           (a ,a)
           (t ,t)
           (U (seq i 0 t (f (/ i t) u a)))
           (U~ (map real-part (DFT U)))
           (U^ (map (lambda (y) (/ (* y (length u)) t)) U~))
           (w ,w)
           (pad ,pad)
           (y1 (apply min (append U U^)))
           (y2 (apply max (append U U^)))
           (dx1 (- (- (- w) 1.0) pad))
           (dx2 2.0))
     ($graphical
      ($axes (- -0.5 pad) (+ w pad 0.5)
             (- y1 0.5) (+ y2 0.5) dx1)
      ($pen-color "dark grey"
        ($line ($point (+ w dx1) (- y1 0.5))
               ($point (+ w dx1) (+ y2 0.5))))
      ($pen-color "magenta"
        ($smoothed u a w pad dx1))
      ($pen-color "red"
        ($points U w dx1)))))
